// Balloon.cpp : implementation file
//

#include "stdafx.h"
#include "Balloon.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBalloon

CBalloon::CBalloon()
{
}

CBalloon::~CBalloon()
{
}


BEGIN_MESSAGE_MAP(CBalloon, CWnd)
        //{{AFX_MSG_MAP(CBalloon)
        ON_WM_ERASEBKGND()
        ON_WM_PAINT()
        ON_WM_CREATE()
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CBalloon message handlers


/****************************************************************************
*                          CBalloon::GetBalloonRect
* Inputs:
*       CRect & r: Desired rectangle
* Result: void
*       
* Effect: 
*       Gets the balloon rectangle
****************************************************************************/

void CBalloon::GetBalloonRect(CRect & r)
    {
     GetClientRect(&r);
     r.bottom = (int) ((double)r.Height() * 0.75);
     r.right =  (int) ((double)r.Width() * 0.80);
    } // CBalloon::GetBalloonRect


BOOL CBalloon::Create(LPCTSTR lpszWindowName, LPCTSTR fontname, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, CCreateContext* pContext) 
   {
    LPCTSTR BalloonClassName = AfxRegisterWndClass(0, 0, (HBRUSH)(COLOR_INFOBK + 1), 0);
    
    ASSERT((dwStyle & WS_CHILD) == 0); // must not be child style
    dwStyle |= WS_POPUP; // must be popup
        

    if(!CWnd::CreateEx(0, BalloonClassName, lpszWindowName, dwStyle, rect, pParentWnd, 0, pContext))
        return FALSE;

    CFont * f = pParentWnd->GetFont();
    LOGFONT lf;
    f->GetLogFont(&lf);
    lf.lfHeight *= 2;
    lstrcpy(lf.lfFaceName, fontname);
    font.CreateFontIndirect(&lf);

    return TRUE;
   }

BOOL CBalloon::OnEraseBkgnd(CDC* pDC) 
   {
    //CRect r;
    //GetClientRect(&r);
    //pDC->FillSolidRect(&r, RGB(255, 255, 0));
    //return TRUE;
    return CWnd::OnEraseBkgnd(pDC);
   }

void CBalloon::OnPaint() 
   {
    CPaintDC dc(this); // device context for painting

    CRgn rgn;
    GetBalloonRgn(rgn);

    CBrush br;
    br.CreateStockObject(BLACK_BRUSH);

    dc.FrameRgn(&rgn, &br, ::GetSystemMetrics(SM_CXBORDER), ::GetSystemMetrics(SM_CYBORDER));
        
    CString text;
    GetWindowText(text);
    
    CRect r;
    GetBalloonRect(r);

    dc.SelectObject(&font);
    CSize sz = dc.GetTextExtent(text);
    
    dc.SetBkMode(TRANSPARENT);
    dc.SetTextColor(::GetSysColor(COLOR_INFOTEXT));
    dc.SetTextAlign(TA_CENTER);
    dc.TextOut(r.Width() / 2,
               r.Height() / 2 - sz.cy / 2,
               text);
        
    // Do not call CWnd::OnPaint() for painting messages
   }

/****************************************************************************
*                           CBalloon::PostNcDestroy
* Result: void
*       
* Effect: 
*       Deletes the dynamic intance of the CWnd * object
****************************************************************************/

void CBalloon::PostNcDestroy() 
   {
    CWnd::PostNcDestroy();
    delete this;
   }

int CBalloon::OnCreate(LPCREATESTRUCT lpCreateStruct) 
   {
    if (CWnd::OnCreate(lpCreateStruct) == -1)
       return -1;
        
    CRgn rgn;
    GetBalloonRgn(rgn);

    SetWindowRgn((HRGN)rgn, TRUE);
    rgn.Detach();
    
    return 0;
}

/****************************************************************************
*                           CBalloon::GetBalloonRgn
* Inputs:
*       CRgn & windowrgn: A CRgn object which has no region attached
* Result: void
*       
* Effect: 
*       Intializes the region to the balloon region
****************************************************************************/

void CBalloon::GetBalloonRgn(CRgn & windowrgn)
    {
     CRect empty;
     empty.SetRectEmpty();
     windowrgn.CreateRectRgnIndirect(&empty);

     CRect r;
     GetBalloonRect(r);

     CClientDC dc(this);
     dc.BeginPath();
     dc.Ellipse(&r);
     dc.EndPath();

     CRgn rgn;
     rgn.CreateFromPath(&dc);


     windowrgn.CombineRgn(&windowrgn, &rgn, RGN_OR);

     rgn.DeleteObject();

     CRect client;
     GetClientRect(&client);
     dc.BeginPath();
     dc.MoveTo(r.Width() / 2, r.Height() / 2);
     dc.LineTo(client.right, client.bottom);
     dc.LineTo(r.Width() / 2, r.Height());
     dc.CloseFigure();
     dc.EndPath();

     rgn.CreateFromPath(&dc);
     windowrgn.CombineRgn(&windowrgn, &rgn, RGN_OR);

     rgn.DeleteObject();
    } // CBalloon::GetBalloonRgn
